/*
 * Ambient - the ultimate desktop
 * ------------------------------
 * Copyright 2001-2005 by David Gerber <zapek@morphos.net>
 * All Rights Reserved
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * $Id: loadimage.c,v 1.7 2013/10/28 12:25:41 geit Exp $
 */

#include "ambient.h"

/* public */

/* private */
#include "loadimage.h"
#include "datatypes_picture.h"
#include "mui_func.h"
#include "methodstack.h"
#include "gfx_scale.h"
#include "gfx_bitmap.h"
#include "threads.h"


ULONG tr_loadimage(APTR obj, CONST_STRPTR path)
{
	ULONG rc = FALSE;
	
	THREAD;
	CHECKOBJECT(obj);

	if (path && *path)
	{
		APTR dtp;
		LONG err = 0;

		if ( (dtp = datatypes_picture_create(path,
			DTPTAG_ARGB32, TRUE,
			DTPTAG_ErrorPtr, &err,
			TAG_DONE)) )
		{

			if (((ULONG)datatypes_picture_getattr(dtp, DTP_WIDTH) < 32768) && ((ULONG)datatypes_picture_getattr(dtp, DTP_HEIGHT) < 32768))
			{
				rc = TRUE;
				methodstack_push_sync(obj, 2, MM_Imageview_AddImage, dtp);
			}
			else
			{
				/* XXX: picture too big.. tell so */
				datatypes_picture_delete(dtp);
			}
		}
		else
		{
			methodstack_push_sync(obj, 2, MM_Imageview_AddError, err);
		}
	}
	return threads_check_abort() ? ABORTED : rc;
}


ULONG tr_scaleimage(APTR obj, APTR dtp, ULONG width, ULONG height)
{
	struct BitMap *bm;

	THREAD;
	CHECKOBJECT(obj);
	ASSERT(dtp);
	ASSERT(width);
	ASSERT(height);

	if ( (bm = gfx_bitmap_create(width, height, 32, BITMAPTAG_Format, BITMAPVAL_Format_ARGB32, TAG_DONE)) )
	{
		if (gfx_scale(datatypes_picture_getattr(dtp, DTP_BITMAP), bm, width, height,
			SCALETAG_Bilinear, TRUE,
		TAG_DONE))
		{
			methodstack_push_sync(obj, 2, MM_Imageview_AddImageScaled, bm);
			return threads_check_abort() ? ABORTED : TRUE;
		}
		gfx_bitmap_delete(bm);
	}
	return threads_check_abort() ? ABORTED : FALSE;
}

/* XXX: maybe add scaling stuff! */
